
{
Jana 2 Remote Class
by Ulf Schaper (UlfSchaper@gmx.de)

Achtung! Auf den Jana2-Release-Preview getrimmt!

Thanks to T. Hauck (www.janaserver.de, T.Hauck@t-online.de),
          Ekkehard Domning (DomIS Internet Solution, www.domis.de, edo@domis.de),
          Thanks to Michael, cagbln@t-online.de who finds out the technical Details.
          *.* :)
}

unit jana2remote;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  IdTCPConnection, IdTCPClient, IdBaseComponent, IdComponent, IdUDPBase,
  IdUDPClient, commdefs, idHTTP;

const
  IntegerError = -5;
  StringError  = 'Fehler';
  BooleanError = false;
  BooleanSuccess = true;

  Jana2DefaultPort = 4295;


type
  TIpType = (itTCP,itUDP);

  rsTcpInfo = record
    NumberOfConnections:integer;
    OnlineSeconds:integer;
    SecondsToDisconnect:integer;
    IP:string;
  end;


type
  TJana2RemoteErrorEvent = procedure(Sender: TObject; Text:string) of object;
  EJana2RemoteBeschaeftigtException = Exception;
const
  eUDPinUse = 'Komponente ist beschftigt - keine erneute UDP-Verbindung';
  eTCPinUse = 'Komponente ist beschftigt - keine erneute TCP-Verbindung';

type
  TJana2Remote = class(TComponent)
  private

    FHost:string;
    FPort:integer;
    FOnError:TJana2RemoteErrorEvent;
    IndyTCP : TIdTCPClient;
    IndyUDP : TIdUDPClient;
    IndyHTTP: TIdHTTP;
    AnzahlDerAbfragen:word;
    procedure PublishError(Text:string);
    procedure InnerSendUdp(CommandNr:byte);

  protected { Protected-Deklarationen }

    LetzteTCPAbfrage:rsTcpInfo;

    procedure SetHostAndPort(ClientType:TIpType);
    procedure RefreshTCPStructure;
    procedure SendUdp(CommandNr:byte);
    function  GetUdp(CommandNr:Byte):string;

    function ReadHost:string;
    function ReadPort:integer;
    procedure WriteHost(New:string);
    procedure WritePort(New:integer);

    procedure TCPClientConnected(Sender: TObject);

  public { Public-Deklarationen }

    UDPBeschaeftigt:boolean;
    TCPBeschaeftigt:boolean;

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    function GetLockState:byte; // "Gesperrt" abfragen, ls_locked oder ls_unlocked
    function GetConnectionState:byte; // cs_Online, wenn Verbindung aufgebaut wird oder
                                         //  vorhanden ist, sonst cs_offline oder cs_error
    function GetConfigPort:integer; // Gibt den Port zurck, ber den die
                                 // Konfiguration verfgbar ist.
    function GetNumberOfConnections(Refresh:boolean):integer; // Gibt die Anzahl der Verbindungen des
                                             // Proxys zurck
    function GetOnlineSeconds(Refresh:boolean):integer; // Onlinezeit seit Programmstart
    function GetSecondsToDisconnect(Refresh:boolean):integer; // "Timeout", d.h. Anzahl der Sekunden,
                                             // bis die Verbindung zwangsweise
                                             // getrennt wird.
    function GetDialUpIP(Refresh:boolean):string; // IP der DF-Verbindung, falls keine Verbindung
                                 // vorhanden ist, '0.0.0.0'
    function SwitchLockState:boolean; // Sperrt- oder Entsperrt.
    function FetchMails:boolean; // Aktiviert die EMail-Abholung
    function ShutdownJana:boolean; // Beendet Jana
    function RestartJana(Benutzername,Passwort:string):boolean; // Startet Jana neu.
    function Logout:boolean; // Loggt den Benutzer aus. Falls er der letzte ist,
                             // wird die Verbindung beendet.
  published
    { Published-Deklarationen }
    property Port : integer read ReadPort write WritePort default Jana2DefaultPort;
    property Host : string read ReadHost write WriteHost;
    property OnError : TJana2RemoteErrorEvent read FOnError write FOnError;
  end;

procedure Register;

implementation

// Grundfunktionen
procedure Register;
begin
  RegisterComponents('Sonstiges', [TJana2Remote]);
end;

constructor TJana2Remote.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  IndyTCP := TIdTcpClient.Create(self);
  IndyUDP := TIdUdpClient.Create(self);
  IndyHTTP := TIdHTTP.Create(self);
  AnzahlDerAbfragen := 0;
  WriteHost('');
  WritePort(Jana2DefaultPort);
end;

destructor TJana2Remote.Destroy;
begin
  FreeAndNil(IndyTCP);
  FreeAndNil(IndyUDP);
  FreeAndNil(IndyHTTP);  
  inherited Destroy;
end;

function TJana2Remote.ReadHost:string;
begin
 result := FHost;
end;

function TJana2Remote.ReadPort:integer;
begin
 Result := FPort;
end;

procedure TJana2Remote.WriteHost(New:string);
begin
 FHost := New;
end;

procedure TJana2Remote.WritePort(New:integer);
begin
 // Evtl. Sicherheitsabfragen wie "if new <> 0 then" einfgen
 FPort := New;
end;

// Kommunikationsfunktionen

procedure TJana2Remote.SetHostAndPort(ClientType:TIpType);
begin
try
 if ClientType = itTCP then begin
  IndyTCP.Host := ReadHost;
  IndyHTTP.Host := ReadHost;
  IndyTCP.Port := ReadPort;
 end else begin
  IndyUDP.Host := ReadHost;
  IndyUDP.Port := ReadPort;
  IndyUDP.ReceiveTimeout := 5000;
 end;
except
  On E:Exception do PublishError(E.Message);
end;
end;

procedure TJana2Remote.InnerSendUdp(CommandNr:byte);
var
   MeinTRRequest:tagRemoteRequest;

begin
  SetHostAndPort(itUDP);

  if AnzahlDerAbfragen = High(Word) then AnzahlDerAbfragen := 0;
  inc(AnzahlDerAbfragen);

  MeinTRRequest.sRequestId := AnzahlDerAbfragen;
  MeinTRRequest.cCmd := CommandNr;

  IndyUDP.SendBuffer(MeinTRRequest,SizeOf(MeinTRRequest));
end;

procedure TJana2Remote.SendUdp(CommandNr:byte);
begin
 try
  if UDPBeschaeftigt then Raise EJana2RemoteBeschaeftigtException.Create(eUDPinUse);
  UDPBeschaeftigt := true;
  InnerSendUdp(CommandNr);
  UDPBeschaeftigt := false;
 except
  On E:Exception do PublishError(E.Message);
 end;
end;

function TJana2Remote.GetUdp(CommandNr:Byte):string;
 var MeinTRAnswer:tagRemoteAnswer;

begin
 try
  if UDPBeschaeftigt then Raise EJana2RemoteBeschaeftigtException.Create(eUDPinUse);
  UDPBeschaeftigt := true;
  IndyUDP.ReceiveString(3); // Dies soll den Puffer leeren
  InnerSendUdp(CommandNr);
  IndyUDP.ReceiveBuffer(MeinTRAnswer, sizeOf(tagRemoteAnswer), 500);

  if (MeinTRAnswer.sRequestId = AnzahlDerAbfragen) and (MeinTRAnswer.Status = 1) then
    begin
      SetLength(result,sizeof(MeinTRAnswer.Data));
      StrLCopy(pchar(result),MeinTRAnswer.Data,MeinTRAnswer.DataSize);
    end else
      result := '';

  UDPBeschaeftigt := false;
 except
  On E:Exception do PublishError(E.Message);
 end;
end;

procedure TJana2Remote.RefreshTCPStructure;
begin
 try
  if TCPBeschaeftigt then Raise EJana2RemoteBeschaeftigtException.Create(eTCPinUse);
  TCPBeschaeftigt := true;
  SetHostAndPort(itTCP);
  IndyTCP.OnConnected := TCPClientConnected;
  IndyTCP.Connect;
 except
  On E:Exception do PublishError(E.Message);
 end;
end;

procedure TJana2Remote.TCPClientConnected(Sender: TObject);
var MeintagStatus:tagStatus;
    MeintagLog:tagLog;
    Datentyp:integer;
begin
try
 try
  // tcp-Verbindung etabliert, jetzt werden die Daten abgerufen

  IndyTCP.WriteLn(inttostr(byte(ssTcpInfo)));

   Datentyp := indytcp.ReadInteger(true);
   case DatenTyp of
    tagStatusIType:
      begin
          IndyTCP.ReadBuffer(MeintagStatus.rNrConnections,sizeOf(tagStatus)-sizeof(MeintagStatus.iType));
          LetzteTCPAbfrage.NumberOfConnections :=
                   FourBytesSwap(MeintagStatus.rNrConnections);
          LetzteTCPAbfrage.OnlineSeconds :=
                   FourBytesSwap(MeintagStatus.iTotalRasTime);
          LetzteTCPAbfrage.SecondsToDisconnect :=
                   FourBytesSwap(MeintagStatus.iTimeUntilDisconnect);
          LetzteTCPAbfrage.IP :=
                   MeintagStatus.caRasIP;
      end;
    tagLogIType:
      begin
          // Sollte nicht auftreten
          IndyTCP.ReadBuffer(MeintagLog.iNrOfStrings,sizeOf(tagLog)-sizeof(MeintagLog.iType));
          ShowMessage(MeinTagLog.caStrings);
      end;
    else ShowMessage('Unbekannter Datentyp');
   end;

 finally
  IndyTCP.Disconnect;
  TCPBeschaeftigt := false;
 end;
except
 On E:Exception do PublishError('TCP-Fehler: '+E.Message);
end;
end;

// ffentliche Funktionen
function TJana2Remote.GetLockState:byte;
var ReturnedString:string;
begin
result := 255;
try
 ReturnedString := GetUDP(Byte(ssUdpLockState));
 if Length(ReturnedString) >= 1 then
   result := Byte(ReturnedString[1]);
 // ls_unlocked bei 0, ls_locked bei 1.
except
  On E:Exception do PublishError(E.Message);
end;
end;

function TJana2Remote.GetConnectionState:byte;
var ReturnedString:string;
begin
result := 255;
try
 ReturnedString := GetUDP(byte(ssUdpRasState));
 if Length(ReturnedString) >= 1 then
   result := Byte(ReturnedString[1]);
 // cs_Offline = 0, cs_Online = 1, cs_error = 2
except
  On E:Exception do PublishError(E.Message);
end;
end;

function TJana2Remote.GetConfigPort:integer;
var ReturnedString:string;
begin
result := 2506;
try
 ReturnedString := GetUDP(byte(ssUdpConfigPort));
 if Length(ReturnedString) >= 2 then
    result := (Byte(ReturnedString[2])*256) OR Byte(ReturnedString[1]);
except
  On E:Exception do PublishError(E.Message);
end;
end;

function TJana2Remote.GetNumberOfConnections(Refresh:boolean):integer;
begin
if Refresh then RefreshTCPStructure;
Result := LetzteTCPAbfrage.NumberOfConnections;
end;

function TJana2Remote.GetOnlineSeconds(Refresh:boolean):integer;
begin
if Refresh then RefreshTCPStructure;
Result := LetzteTCPAbfrage.OnlineSeconds;
end;

function TJana2Remote.GetSecondsToDisconnect(Refresh:boolean):integer;
begin
if Refresh then RefreshTCPStructure;
Result := LetzteTCPAbfrage.SecondsToDisconnect;
end;

function TJana2Remote.GetDialUpIP(Refresh:boolean):string;
begin
result := StringError;
if Refresh then RefreshTCPStructure;
Result := LetzteTCPAbfrage.IP;
end;

function TJana2Remote.SwitchLockState:boolean;
begin
 try
  result := BooleanSuccess;
  SendUdp(byte(ssUdpLock));
 except
  result := BooleanError;
 end;
end;

function TJana2Remote.FetchMails:boolean;
begin
 try
  result := BooleanSuccess;
  SendUdp(byte(ssUdpMail));
 except
  result := BooleanError;
 end;
end;

function TJana2Remote.Logout:boolean;
begin
 try
  result := BooleanSuccess;
  SendUdp(byte(ssUdpDisconnect));
 except
  result := BooleanError;
 end;
end;


function TJana2Remote.ShutdownJana:boolean;
begin
 try
  result := BooleanSuccess;
  SendUdp(byte(ssUdpServerShutdown));
 except
  result := BooleanError;
 end;
end;

function TJana2Remote.RestartJana(Benutzername,Passwort:string):boolean;
var RequestString: string;
begin
 try
  result := BooleanSuccess;
  IndyHTTP.Request.Username := Benutzername;
  IndyHTTP.Request.Password := Passwort;
  IndyHTTP.Host := Host;
  IndyHTTP.Port := GetConfigPort;
  RequestString := 'http://'+Host+':'+inttostr(IndyHTTP.Port)+'/jana-admin/admin/?Restart=true';
//  ShowMessage(RequestString);
  IndyHTTP.Get(RequestString);
 except
  result := BooleanError;
 end;
end;

procedure TJana2Remote.PublishError(Text:string);
begin
  If Assigned(FOnError) then
    FOnError(self,text);
end;

end.

